package polyglot.ext.ibex;

import java.io.Reader;
import java.util.List;

import polyglot.ast.NodeFactory;
import polyglot.ext.ibex.ast.IbexNodeFactory;
import polyglot.ext.ibex.ast.IbexNodeFactory_c;
import polyglot.ext.ibex.parse.Grm;
import polyglot.ext.ibex.parse.Lexer_c;
import polyglot.ext.ibex.visit.Rewriter;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.visit.*;
import polyglot.frontend.*;
import polyglot.frontend.Compiler;
import polyglot.lex.Lexer;
import polyglot.types.TypeSystem;
import polyglot.util.ErrorQueue;

/**
 * Extension information for Ibex extension.
 */
public class ExtensionInfo extends polyglot.ext.jl.ExtensionInfo {
    public String[] defaultFileExtensions() {
        return new String[] { "ibex", "java", "jl" };
    }

    public String compilerName() {
        return "ibex";
    }

    public Parser parser(Reader reader, FileSource source, ErrorQueue eq) {
        Lexer lexer = new Lexer_c(reader, source.name(), eq);
        Grm grm = new Grm(lexer, ts, nf, eq);
        return new CupParser(grm, source, eq);
    }

    protected NodeFactory createNodeFactory() {
        return new IbexNodeFactory_c();
    }

    protected TypeSystem createTypeSystem() {
        return new IbexTypeSystem_c();
    }

    public static final Pass.ID REWRITE = new Pass.ID("rewrite");

    public List passes(Job job) {
      List l = super.passes(job);

      beforePass(l, Pass.OUTPUT,
                    new VisitorPass(REWRITE, job,
                                    new Rewriter(job, 
                                                 (IbexTypeSystem)typeSystem(), 
                                                 (IbexNodeFactory)nf)));
      return l;
  }
}
