package polyglot.ext.ibex.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.visit.*;
import polyglot.util.*;
import java.util.*;

/**
 * Tag interface for elements of a rule RHS.
 */
public abstract class RhsNode_c extends Term_c implements RhsNode {
    Block action;
    Rhs rhs;

    public RhsNode_c(Position pos, Block action) {
        super(pos);
        this.action = action;
    }

    public Block action() { return action; }
    public RhsNode action(Block action) {
        RhsNode_c n = (RhsNode_c) copy();
        n.action = action;
        return n;
    }

    public Rhs rhs() { return rhs; }
    public RhsNode rhs(Rhs rhs) {
        RhsNode_c n = (RhsNode_c) copy();
        n.rhs = rhs;
        return n;
    }

    public abstract Node visitChildren(NodeVisitor v);

    public Node buildTypes(TypeBuilder tb) throws SemanticException {
        IbexTypeSystem ts = (IbexTypeSystem) tb.typeSystem();
        return makeRhs(ts);
    }

    protected abstract Node makeRhs(IbexTypeSystem ts);

    /** Build type objects for the method. */
    public NodeVisitor disambiguateEnter(AmbiguityRemover ar) throws SemanticException {
        if (ar.kind() == AmbiguityRemover.SUPER) {
            return ar.bypassChildren(this);
        }
        else if (ar.kind() == AmbiguityRemover.SIGNATURES) {
            if (action != null) {
                return ar.bypass(action);
            }
       }
        
        return ar; 
    }   

    public Node disambiguate(AmbiguityRemover ar) throws SemanticException {
        IbexTypeSystem ts = (IbexTypeSystem) ar.typeSystem();
        return makeRhs(ts);
    }

    public Context enterScope(Context c) {
        return c.pushBlock();
    }

    /**
     * Return the first (sub)term performed when evaluating this
     * term.
     */
    public Term entry() {
        return this;
    }

    /**
     * Visit this term in evaluation order.
     */
    public List acceptCFG(CFGBuilder v, List succs) {
        return succs;
    }

    public Node rewrite(Rewriter rw) {
        return this;
    }
}
