package polyglot.ext.ibex.lr;

import polyglot.ext.ibex.types.*;
import polyglot.util.*;
import java.util.*;
import polyglot.main.Report;

public class LREncoding {
    GLR glr;

    int[][] outputActionTable;
    int[][] outputGotoTable;
    int[]   outputOverflowTable;
    int[]   outputRuleTable;

    public LREncoding(GLR glr) {
        this.glr = glr;
        encodeOutputTables();
    }

    protected void encodeOutputTables() {
        if (Report.should_report(TOPICS, 1))
            Report.report(1, "encoding tables");

        int[][] gotoTable = glr.lrc.gotoTable;
        Set[][] actionTable = glr.lrc.actionTable;
        List states = glr.lrc.states;
        List terminals = glr.terminals;
        List nonterminals = glr.nonterminals;

        outputActionTable = new int[states.size()][terminals.size()];
        List overflow = new ArrayList();

        for (int i = 0; i < actionTable.length; i++) {
            Set[] actionRow = actionTable[i];
            ROW:
            for (int j = 0; j < actionRow.length; j++) {
                if (actionRow[j].size() == 1) {
                    for (Iterator k = actionRow[j].iterator(); k.hasNext(); ) {
                        Action a = (Action) k.next();
                        outputActionTable[i][j] = a.encode();
                    }
                }
                else if (actionRow[j].size() > 1) {
                    outputActionTable[i][j] = Action.encode(Action.OVERFLOW, overflow.size());
                    // add the number of actions
                    overflow.add(new Integer(actionRow[j].size()));

                    // now add the actions.
                    for (Iterator k = actionRow[j].iterator(); k.hasNext(); ) {
                        Action a = (Action) k.next();
                        overflow.add(new Integer(a.encode()));
                    }
                }
            }
        }

        outputOverflowTable = new int[overflow.size()];

        for (int j = 0; j < overflow.size(); j++) {
            Integer n = (Integer) overflow.get(j);
            outputOverflowTable[j] = n.intValue();
        }

        outputGotoTable = gotoTable;

        List rules = glr.rules;

        outputRuleTable = new int[rules.size()];

        for (int j = 0; j < rules.size(); j++) {
            GLRRule r = (GLRRule) rules.get(j);
            outputRuleTable[j] = r.encode();
        }
    }

    public String[] encodedActionTable() {
        if (outputActionTable == null) {
            throw new InternalCompilerError("tables not yet encoded");
        }
        return new Encoder().encode(outputActionTable);
    }

    public String[] encodedOverflowTable() {
        if (outputOverflowTable == null) {
            throw new InternalCompilerError("tables not yet encoded");
        }
        return new Encoder().encode(outputOverflowTable);
    }

    public String[] encodedGotoTable() {
        if (outputGotoTable == null) {
            throw new InternalCompilerError("tables not yet encoded");
        }
        return new Encoder().encode(outputGotoTable);
    }

    public String[] encodedRuleTable() {
        if (outputRuleTable == null) {
            throw new InternalCompilerError("tables not yet encoded");
        }
        return new Encoder().encode(outputRuleTable);
    }

    static Collection TOPICS = Arrays.asList( new String[] { "lr", "ibex" });
}
