package polyglot.ext.ibex.types;

import polyglot.types.*;
import polyglot.ext.jl.types.TypeSystem_c;
import polyglot.util.*;
import polyglot.frontend.Source;
import java.util.*;

public class IbexTypeSystem_c extends TypeSystem_c implements IbexTypeSystem {
    public ParsedClassType createClassType(LazyClassInitializer init, Source fromSource) {
        return new MutableParserType_c(this, init, fromSource);
    }

    public Nonterminal nonterminal(Position pos, ParserType container, Flags flags, Type type, String name, List choices, boolean isStart) {
        return new Nonterminal_c(this, pos, container, flags, type, name, choices, isStart);
    }

    public RhsCase rhsCase(Position pos, List elements, List tags) {
        return new RhsCase_c(this, pos, elements, tags);
    }

    public RhsMerge rhsMerge(Position pos, RhsCase rhs1, RhsCase rhs2) {
        return new RhsMerge_c(this, pos, rhs1, rhs2);
    }

    public Terminal terminal(Position pos, ParserType container, Flags flags, Type type, String name) {
        return new Terminal_c(this, pos, container, flags, type, name);
    }

    public ClassType runtimeMergeableType() {
        return load("ibex.runtime.Mergeable");
    }

    public ClassType runtimeParserType() {
        return load("ibex.runtime.Parser");
    }

    public ClassType runtimeGLRParserType() {
        return load("ibex.runtime.GLRParser");
    }

    public ClassType runtimeParserImplType() {
        return load("ibex.runtime.ParserImpl");
    }

    public ClassType runtimeTerminalType() {
        return load("ibex.runtime.Terminal");
    }

    public Symbol findSymbol(ParserType container, String name) throws SemanticException {
        Symbol s;
        
        s = container.nonterminalNamed(name);

        if (s != null) {
            return s;
        }

        s = container.terminalNamed(name);

        if (s != null) {
            return s;
        }

        Type t = container.superType();

        if (t instanceof ParserType) {
            try {
                return findSymbol((ParserType) t, name);
            }
            catch (SemanticException e) {
            }
        }

        throw new SemanticException("Could not find symbol \"" + name +
                                    "\" in class \"" + container + "\".");
    }
}
