package polyglot.ext.j0.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.Flags;
import polyglot.types.Package;
import polyglot.types.Type;
import polyglot.types.Qualifier;
import polyglot.util.*;
import polyglot.ext.j0.ExtensionInfo;
import polyglot.ext.jl.qq.*;
import java.util.*;

/**
 * NodeFactory for the j0 extension.
 */
public class J0NodeFactory_c extends NodeFactory_c implements J0NodeFactory {
    private final J0ExtFactory extFactory;
    private final J0DelFactory delFactory;

     public J0NodeFactory_c() {
	 super(new J0ExtFactory_c() {}, new J0DelFactory_c() {}); 
	 extFactory = (J0ExtFactory)extFactory();
	 delFactory = (J0DelFactory)delFactory();
     }


    // TODO:  Implement factory methods for new AST nodes.

    /** Create a J0 SourceFile node. Requires extra arguments to aid in
     * inference of the implied J0 structures. 
     * @param fpos the position of the sourcefile.
     * @param l the list of j0 top level declarations, as defined in the j0 grammar
     * @param a the package node 
     * @param b the imports node
     * @param ext ExtensionInfo (needed for quasi-quoting)
     * @param fileName the file name of the sourcefile*/
    public SourceFile SourceFile(Position fpos, PackageNode a, List b, List l, ExtensionInfo ext, String fileName) {

	QQ quasiQuoter = new QQ(ext);

	TypedList tList = new TypedList(new LinkedList(), TopLevelDecl.class, false);

	TypedList mList = new TypedList(new LinkedList(), ClassMember.class, false);

	TypedList sList = new TypedList(new LinkedList(), Stmt.class, false);
	ListIterator topIterator = l.listIterator();

	while(topIterator.hasNext()) {
	    Object n = topIterator.next();

	    if (n instanceof ClassDecl) {
		tList.add(n);
	    } else if (n instanceof MethodDecl) {
		mList.add(n);
	    } else if (n instanceof List) {
		mList.addAll((List)n);
	    } else if (n instanceof Stmt) {
		sList.add(n);
	    }
	    
	}

	if(sList.isEmpty() && mList.isEmpty()) {
	    return SourceFile(fpos, a, b, tList);
	} else {
	    // create main method
	    // create method header
	    Position pos = Position.COMPILER_GENERATED;

	    Block bodyBlock = Block(pos, sList);
	    ClassMember md = quasiQuoter.parseMember("public static void main(String[] args) {%S}", bodyBlock);						     

	    // append main method to mList
	    mList.add(md);

	    // create class from mList

	    int slash = fileName.lastIndexOf("/");
	    int dot = fileName.lastIndexOf(".");
	    
	    String className = fileName.substring(slash + 1, dot);
	    
	    // make other classes nested
	    mList.addAll(tList);

	    ClassDecl cd = quasiQuoter.parseDecl("public class %s {%LM}", className, mList);

	    // add class to a new tList
	    TypedList retList = new TypedList(new LinkedList(), TopLevelDecl.class, false);
	    retList.add(cd);
	    return SourceFile(fpos, a, b, retList);
	}
    }

    public Call Call(Position pos, Receiver target, String name, List args) {
        Call n = new J0Call_c(pos, target, name, args);
        n = (Call)n.ext(extFactory.extCall());
        n = (Call)n.del(delFactory.delCall());
        return n;
    }



    // TODO:  Override factory methods for overriden AST nodes.
    // TODO:  Override factory methods for AST nodes with new extension nodes.

    public ImportStaticMethod ImportStaticMethod(Position pos, String name, List parameters) {
        ImportStaticMethod n = new ImportStaticMethod_c(pos, name, parameters);
        n = (ImportStaticMethod)n.ext(extFactory.extImportStaticMethod());
        n = (ImportStaticMethod)n.del(delFactory.delImportStaticMethod());
        return n;
    }
   
}
