package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.visit.*;
import polyglot.ext.polyj.util.*;

import polyglot.ext.jl.ast.*;

import java.util.*;

public class PolyJMethodDecl_c extends MethodDecl_c implements PolyJMethodDecl {

    protected List constraints;
    
    /**
     * Constructors for PolyJMethodNodes with constraints
     */
    public PolyJMethodDecl_c( Position pos, Flags flags, TypeNode returnType, 
                             String name, List formals, List throwTypes, List constraints, 
                             Block body)
    {
        super( pos, flags, returnType, name, formals, throwTypes, body);
        this.constraints = TypedList.copyAndCheck(constraints, ClassConstraint.class, true);
    }

    public List constraints() {
        return constraints;
    }
    
    public PolyJMethodDecl constraints(List constraints) {
        PolyJMethodDecl_c n = (PolyJMethodDecl_c) copy();
        n.constraints = TypedList.copyAndCheck(constraints, ClassConstraint.class, true);
        return n;
    }
    
    protected PolyJMethodDecl reconstruct(TypeNode returnType, List formals, 
                                          List throwTypes, List constraints,
                                          Block body)
    {
        if (      this.returnType != returnType 
                  || !CollectionUtil.equals(this.formals, formals)
                  || !CollectionUtil.equals(this.throwTypes, throwTypes)
                  || !CollectionUtil.equals(this.constraints, constraints)
                  || this.body != body) {
            PolyJMethodDecl_c n = (PolyJMethodDecl_c) super.reconstruct(returnType, formals, throwTypes, body);
            n.constraints = TypedList.copyAndCheck(constraints, ClassConstraint.class, true);
            return n;
        } else {
            return this;
        }
    }

    public Node visitChildren(NodeVisitor v) 
    {
        TypeNode newReturns = (TypeNode) visitChild(returnType, v);
        List newFormals = visitList(formals, v);
        List newConstraints = visitList(constraints, v);
        List newExceptions = visitList(throwTypes, v);
        Block newBody = (Block) visitChild(body, v);
        return reconstruct(newReturns, newFormals, newExceptions, newConstraints, newBody);
    }

    public void dump(CodeWriter w) 
    {
        w.write( "POLYJ METHOD");
        w.write( " < " + name + " >");
        w.write( " < " + flags.toString() + "> ");
    }
    
    public void prettyPrint(CodeWriter w, PrettyPrinter pp) {
        // FIXME: print constraints
        super.prettyPrint(w, pp);
    }

    public void translate(CodeWriter w, Translator tr) {
        Context c = tr.context();
	PolyJTypeSystem ts = (PolyJTypeSystem) tr.typeSystem();
	
        w.begin(0);
        w.write(flags.translate());

	ReturnTypeMutator m = new ReturnTypeMutator(methodInstance());
	TypeNode actualReturn;
	if (m.typeChanges()) {
	    actualReturn = returnType.type(m.translatedType());
	} else {
	    actualReturn = returnType;
	}
	
        actualReturn.del().translate(w, tr);
        w.write(" " + name + "(");
        
        w.begin(0);

        boolean isStatic = flags.isStatic();
	boolean isPoly = ts.isPolymorphicClass(c.currentClass());
        boolean addWhereArg = isStatic && isPoly;
        
	// Add in where object to static method
        if (addWhereArg) {
            w.write(PolyJTranslator.adapterInterfaceShortName(
                c.currentClass()) + " $W");
            if (formals.iterator().hasNext()) {
                w.write(", ");
            }
        }

        for (Iterator iter = formals.iterator(); iter.hasNext(); )
        {
            Formal formal = (Formal) iter.next();
            formal.del().translate(w, tr);
            if (iter.hasNext()) {
                w.write( ",");
                w.allowBreak(0, " ");
            }
        }
        w.end();
        w.write( ")");

        if( !throwTypes.isEmpty()) {
            w.allowBreak(6);
            w.write( "throws ");
            for (Iterator i = throwTypes.iterator(); i.hasNext(); ) {
                TypeNode tn = (TypeNode) i.next();
                tn.del().translate(w, tr);
                if (i.hasNext()) {
                    w.write(","); w.allowBreak(4, " ");
                }
            }
        }
        w.end();
        
      	if (body != null) {
	    printSubStmt(body, w, tr.context(enterScope(c)));
	}
	else {
	    w.write(";");
	} 
        
        w.newline(0);
    }
}
