package polyglot.ext.polyj.parse;

import polyglot.ext.polyj.ast.*;

import polyglot.ast.*;
import polyglot.visit.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.parse.*;

import java.util.*;

/**
 * Base implementation of Sub.  Subclasses need to define their own conversion methods.
 */
public abstract class AbstractSub implements Sub {
    Position pos;
    BaseParser parser;
    PolyJNodeFactory nf;

    public AbstractSub(BaseParser parser, Position pos) {
        this.parser = parser;
        this.nf = (PolyJNodeFactory) parser.nf;
        this.pos = pos;
    }
    
    public Expr toExpr() throws Exception {
        parser.die("Illegal expression.", pos);
        return null;
    }
    
    public TypeNode toType() throws Exception {
        parser.die("Illegal type.", pos);
        return null;
    }
    
    /**
     * Convert each item in <code>ls</code> from a Sub to a ParamNode.
     * The parser will die if the conversion isn't possible.
     */
    public static List toParamList(List ls) throws Exception {
        List lp = new TypedList(new ArrayList(ls.size()), ParamNode.class, false);
        
        for (Iterator i = ls.iterator(); i.hasNext(); ) {
            Sub s = (Sub) i.next();
            ParamNode p = s.toParam();
            lp.add(p);
        }
        
        return lp;
    }
    
}
