package polyglot.ext.polyj.types;

import polyglot.ast.*;
import polyglot.frontend.*;
import polyglot.util.*;
import polyglot.types.*;

import polyglot.ext.polyj.ast.*;

import polyglot.ext.jl.types.*;

import java.io.*;
import java.util.*;

public class PolyDlg_c implements PolyDlg {
    /**
     * The object for which this serves as a delegate.
     */
    protected PolyJClassType type;
    
    /**
     * The parameters of type, stored in the delegate to
     * eliminate the need for redundant implementation
     * of them in every subclass of PolyJClassType.
     */
    protected List /*of ParamType*/ params;
    
    /**
     * The type system that "type" was created in, cached
     * here and cast to a PJTS.
     */
    protected transient PolyJTypeSystem ts;
    
    public PolyDlg_c(PolyJClassType type) {
        this.type = type;                                                     
        params = new TypedList(new LinkedList(), ParamType.class, false);
        ts = (PolyJTypeSystem) type.typeSystem();
    }
    
    public List params() {
        return params;
    }
    
    public ParamType paramNamed(String name) {
        for (Iterator p = params().iterator(); p.hasNext();) {
            ParamType par = (ParamType) p.next();
            if (par.name().equals(name)) {
                return par;
            }
        }
        return null;
    }
    
    public boolean isPolymorphic() {
        return params.size() != 0;
    }
    
    public Type instantiate(List args) throws SemanticException {
        return ts.instantiate(type,    TypedList.copyAndCheck(args, Type.class, true));
    }
    
    public void addParam(String paramName) {
        int index = params.size();
        params.add(ts.ParamType(type.position(), type, paramName, index));
    }

    public void addConstraint(ClassConstraint c) throws SemanticException {
        String paramName = c.name();
        ParamType param = paramNamed(paramName);
        if (param == null) {
            throw new SemanticException("Unknown parameter " + paramName,
                c.position());
        }
        paramNamed(paramName).addConstraint(c);
    }

}

